/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "ProcessingInstruction.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomProcessingInstruction.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateGdomProcessingInstruction.h"
#include "gdom/PrivateIntGdomProcessingInstruction.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMProcessingInstruction::GDOMProcessingInstruction(PassRefPtr<WebCore::ProcessingInstruction> impl)
    : GDOMNode(impl)
{
}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "ProcessingInstruction.h"
#include <gdom/GdomProcessingInstruction.h>
#include <gdom/PrivateGdomProcessingInstruction.h>
#include <gdom/PrivateIntGdomProcessingInstruction.h>
#include <gdom/GdomProcessingInstructionprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomProcessingInstruction, gdom_processing_instruction, GDOM_TYPE_NODE);

/*
struct _GdomProcessingInstructionPrivate {
    class WebCore::ProcessingInstruction *item;
};

#define GDOM_PROCESSING_INSTRUCTION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_PROCESSING_INSTRUCTION, GdomProcessingInstructionPrivate))
*/
G_END_DECLS
WebCore::ProcessingInstruction *
gdom_processing_instruction_get_private(GdomProcessingInstruction *request)
{
    GDOMProcessingInstruction *res;
    res = static_cast<GDOMProcessingInstruction*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::ProcessingInstruction*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_TARGET,
    PROP_DATA,
    PROP_SHEET,
};


static void gdom_processing_instruction_finalize(GObject* object)
{
    /*GdomProcessingInstruction* request = GDOM_PROCESSING_INSTRUCTION(object);*/
    /*GdomProcessingInstructionPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_NODE));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_processing_instruction_parent_class)->finalize(object);
}

static void gdom_processing_instruction__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomProcessingInstruction *self = GDOM_PROCESSING_INSTRUCTION(object);
    WebCore::ProcessingInstruction * gself = (gdom_processing_instruction_get_private(self));
    switch (prop_id) {
     case PROP_DATA:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setData(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_processing_instruction__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomProcessingInstruction *self = GDOM_PROCESSING_INSTRUCTION(object);
    WebCore::ProcessingInstruction * gself = (gdom_processing_instruction_get_private(self));
    switch (prop_id) {
    case PROP_TARGET:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->target()));
         break;
    }
    case PROP_DATA:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->data()));
         break;
    }
    case PROP_SHEET:
    {
         PassRefPtr<WebCore::StyleSheet> ptr = gself->sheet();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_processing_instruction_class_init(GdomProcessingInstructionClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_processing_instruction_finalize;
    g_klass->set_property = gdom_processing_instruction__set_property_fn;
    g_klass->get_property = gdom_processing_instruction__get_property_fn;

    g_object_class_install_property (g_klass, PROP_TARGET,
            g_param_spec_string ("target", /* name */
                "processing_instruction_target", /* short description */
                "read-only  gchar * ProcessingInstruction.target", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_DATA,
            g_param_spec_string ("data", /* name */
                "processing_instruction_data", /* short description */
                "read-write  gchar * ProcessingInstruction.data", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SHEET,
            g_param_spec_object ("sheet", /* name */
                "processing_instruction_sheet", /* short description */
                "read-only  GdomStyleSheet * ProcessingInstruction.sheet", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_STYLE_SHEET, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomProcessingInstructionPrivate));*/
}

static void gdom_processing_instruction_init(GdomProcessingInstruction* request)
{
    /*GdomProcessingInstructionPrivate* priv = GDOM_PROCESSING_INSTRUCTION_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

